<?php

/**
 * @package     Sven.Bluege
 * @subpackage  com_eventgallery
 *
 * @copyright   Copyright (C) 2005 - 2019 Sven Bluege All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace Svenbluege\Component\Eventgallery\Site\Library\Factory;
use Svenbluege\Component\Eventgallery\Site\Library\FlickrAccount;

defined('_JEXEC') or die();

class FlickrAccountFactory extends AbstractFactory
{
    protected $_accounts;

    public function getAccountById($id) {

        $sets = $this->getAccounts();
        if (isset($sets[$id]))
        {
            return $sets[$id];
        }

        return null;
    }

    /**
     *
     *
     * @return FlickrAccount()[]
     */
    public function getUsableAccounts() {
        return array_filter($this->getAccounts(), function(/** @var FlickrAccount() $account */ $account) {
            return $account->isUsable();
        });
    }

    /**
     * @return FlickrAccount()[]
     */
    public function getAccounts() {

        if ($this->_accounts == null) {

            $db = $this->db;
            $query = $db->getQuery(true);
            $query->select('*');
            $query->from('#__eventgallery_flickr_account');
            $query->order('ordering');
            $db->setQuery($query);
            $items = $db->loadObjectList();

            $this->_accounts = array();


            foreach ($items as $item) {
                /**
                 * @var FlickrAccount() $itemObject
                 */
                $itemObject = new FlickrAccount($item);
                $this->_accounts[$itemObject->getId()] = $itemObject;
            }
        }

        return $this->_accounts;
    }

}
